// InputChoserDialog.java

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.border.*;

class InputChoserDialog extends JDialog implements ActionListener,AdjustmentListener,FocusListener,MouseListener
{
	Toolkit tk = Toolkit.getDefaultToolkit();
	Dimension screenDim = tk.getScreenSize();
	GridBagConstraints gbc = new GridBagConstraints();
	
	JButton okButton = new JButton("Ok"),
		cancelButton = new JButton("Cancel");
	
	JScrollBar actColorScrb = new JScrollBar(Adjustable.HORIZONTAL,0,0,0,255);
	JTextField actColorText = new JTextField("0",4),
		   actColorField = new JTextField(4);
	
	JTextField[][] textfields = new JTextField[8][8],
		       colorfields = new JTextField[8][8];

	JPanel mainPanel = new JPanel(new BorderLayout()),
	       colorChoserPanel = new JPanel(new GridBagLayout()),
	       labelPanel = new JPanel(new GridLayout(8,8,4,4)),
	       textPanel = new JPanel(new GridLayout(8,8)),
	       buttonPanel = new JPanel(),
	       labelAndtextPanel = new JPanel(new GridLayout(1,2));
	       
	boolean okTransfer,noChange;
		
	
	int[][] toManipulate = new int[8][8];
		
	
	InputChoserDialog(JFrame parent)
	{
		super(parent, "Customize your input", true);
		setSize(640,400);
		setLocation((screenDim.width-640)/2,(screenDim.height-480)/2);				
		getContentPane().setLayout(new BorderLayout());
		
		addWindowListener(new WindowAdapter(){public void windowClosing(WindowEvent e) 
						     {noChange = true;dispose();}});		
				
		mainPanel.setBorder(new SoftBevelBorder(BevelBorder.RAISED));	
		actColorField.setBorder(new SoftBevelBorder(BevelBorder.RAISED));
		
		for (int y = 0; y < 8; y++)
		for (int x = 0; x < 8; x++)
		{
			textfields[y][x] = new JTextField(3);
			textfields[y][x].addActionListener(this);
			textfields[y][x].addFocusListener(this);			
			colorfields[y][x] = new JTextField(3);			
			colorfields[y][x].setBorder(new SoftBevelBorder(BevelBorder.LOWERED));
			colorfields[y][x].setEditable(false);
			colorfields[y][x].addMouseListener(this);
			colorfields[y][x].setToolTipText("Left click to set color");
			
			textPanel.add(textfields[y][x]);
			labelPanel.add(colorfields[y][x]);
		}
		
		labelAndtextPanel.add(labelPanel);
		labelAndtextPanel.add(textPanel);
		
		mainPanel.add(labelAndtextPanel,"Center");
				
		m.addToGBL(colorChoserPanel, actColorScrb,  gbc, 0, 0,2,1,100,0,2);
		m.addToGBL(colorChoserPanel, actColorField, gbc, 2, 0,1,1,100,0,2);
		m.addToGBL(colorChoserPanel, actColorText,  gbc, 3, 0,1,1,  0,0,2);
		
		
		actColorScrb.setToolTipText("Adjust scrollbar to change color");
		actColorField.setToolTipText("THIS is the used color");
		actColorText.setToolTipText("Edit textfield to change color [0..255]");
		actColorScrb.setBlockIncrement(20);		
		actColorField.setEditable(false);
		actColorField.setBackground(new Color(0,0,0));
		actColorScrb.addAdjustmentListener(this);
		actColorText.addActionListener(this);
		actColorText.addFocusListener(this);
					
		mainPanel.add(colorChoserPanel,"South");
		
		okButton.setToolTipText("Save pattern and return");
		cancelButton.setToolTipText("Reject pattern and return");
		okButton.addActionListener(this);cancelButton.addActionListener(this);
		buttonPanel.add(okButton); buttonPanel.add(cancelButton);
						
		getContentPane().add(mainPanel,"Center");
		getContentPane().add(buttonPanel,"South");
				
	}

	public boolean showDialog(InputTransfer trans)
	{						
		int c;
		
		for (int y = 0; y < 8; y++)
		for (int x = 0; x < 8; x++)
		{			
			toManipulate[y][x] = trans.matrix[y][x];
			c = trans.matrix[y][x];
			colorfields[y][x].setBackground(new Color(c,c,c));
			textfields[y][x].setText(String.valueOf(c));	
			textfields[y][x].setToolTipText("YEAH! A textfield with content "+String.valueOf(c));
		} // end loop
		
		okTransfer = false;
		show();		
		if (okTransfer)
		{				
			for (int y = 0; y < 8; y++)
			for (int x = 0; x < 8; x++)
				trans.matrix[y][x] = toManipulate[y][x];			
		}
		return okTransfer;
		
	}
//******************************************************************************************************************	
//**                                                 * LISTENERS *                                                **
//******************************************************************************************************************	


	//******************************* ACTION ************************************
		
	public void actionPerformed(ActionEvent evt)
	{
		Object source = evt.getSource();		
		
		if (source instanceof JTextField) 
		{		
			JTextField src = (JTextField)evt.getSource();
			int c;
								
			for (int y = 0; y < 8; y++)									
			for (int x = 0; x < 8; x++)
			{
				if (!(src == textfields[y][x])) continue;
				else { 
				       if (!m.isValid(src.getText().trim(),0,255)) 
				       { src.setText(String.valueOf(toManipulate[y][x]));				       	 
				         src.requestFocus();
				         src.selectAll();
				       }
				       else 
				       { c = Integer.parseInt(src.getText());
				       	 toManipulate[y][x] = c;
				       	 colorfields[y][x].setBackground(new Color(c,c,c));	
				       	 src.setToolTipText("YEAH! A textfield with content "+String.valueOf(c));
				       }
				       break;
				     } // end else
			} // end of for (int x = 0; ...	
		}
		
		if (source == actColorText)
		{
			if (!m.isValid(actColorText.getText().trim(),0,255)) 
			{ actColorText.setText(String.valueOf(actColorScrb.getValue()));}
			else 
			{
			  int c = Integer.parseInt(actColorText.getText());
			  actColorScrb.setValue(c);
			  actColorField.setBackground(new Color(c,c,c));
			}

		} else
		if (source == okButton)
		{
			okTransfer = true;
			noChange = false;
			dispose();
			
		} else
		if (source == cancelButton)
		{
			noChange = true;
			dispose();
		}
		
	} // end actionPerformed
	
	//******************************* ADJUSTMENT ************************************
	
	public void adjustmentValueChanged(AdjustmentEvent evt)
	{
		Object source = evt.getSource();
		
		if (source == actColorScrb)
		{
			int c = actColorScrb.getValue();
			actColorText.setText(String.valueOf(c));
			actColorField.setBackground(new Color(c,c,c));
		}
	} // end adjustmentValueChanged
	
	//******************************* FOCUS ************************************
	
	public void focusGained(FocusEvent evt)
	{
		Object source = evt.getSource();
		((JTextField)source).selectAll();
	}
	
	public void focusLost(FocusEvent evt)
	{				
		Object source = evt.getSource();
		
		if ( (source instanceof JTextField) && (!evt.isTemporary()) )
		{		
			JTextField src = (JTextField)evt.getSource();
			int c;
					
			for (int y = 0; y < 8; y++)									
			for (int x = 0; x < 8; x++)
			{
				if (!(src == textfields[y][x])) continue;
				else { 
				       if (!m.isValid(src.getText().trim(),0,255)) 
				       { src.setText(String.valueOf(toManipulate[y][x])); }
				       else 
				         {
				         	c = Integer.parseInt(src.getText());
				         	toManipulate[y][x] = c;
				         	colorfields[y][x].setBackground(new Color(c,c,c));
				         	src.setToolTipText("YEAH! A textfield with content "+String.valueOf(c));				     	
				         }
				       break;
				     } // end else
			} // end of for (int x = 0; ...				
		}
	} // end focusLost
	
	
	//******************************* MOUSE ************************************
	
	public void mousePressed(MouseEvent evt)
	{
		Object source = evt.getSource();
		Color c = actColorField.getBackground();
		
		for (int y = 0; y < 8; y++)									
		for (int x = 0; x < 8; x++)
		{
			if (!(source == colorfields[y][x])) continue;
			else 
			{ 	
				((JTextField)source).setBackground(c);
				toManipulate[y][x] = c.getGreen(); // Greyscale!! greenValue = redValue = blueValue !!
				textfields[y][x].setText(String.valueOf(c.getGreen()));	
				textfields[y][x].setToolTipText(
				          "YEAH! A textfield with content "+String.valueOf(c.getGreen()));			
			} // end else
					
		}
	}
	
	public void mouseReleased(MouseEvent evt)
	{
		
	}
	
	public void mouseClicked(MouseEvent evt)
	{
		
	}
	
	public void mouseEntered(MouseEvent evt)
	{
		
	}
	
	public void mouseExited(MouseEvent evt)
	{
		
	}

} // end of class
