// TheCalculator.java

import java.math.*;

class TheCalculator
{

TheGUI gui;	
	
int[][] inputSampleToTransform = new int[8][8];	       // Matrix mit den Werten des eingestellten Inputsamples
int[][] coefField = new int[8][8];		       // Matrix mit den "richtigen" Koeffizienten
int[][] userCoefSampleToTransformBack = new int[8][8]; // Matrix mit den Koeffizienten die der User mit den 
					               // Scrollbars/Textfields selber einstellt
int[][] userInputSample = new int[8][8];	       // Matrix mit den Farbwerten, die der User eingestellt hat	
	
String calculationmode = "all";

	
TheCalculator(TheGUI gui)
{
	this.gui = gui;
	
} // end of constructor



// ************************************** Transformation ********************************
public int[][] transformation(int[][] isf) // isf = input sample field
{	
	int[][] cf = new int[8][8];		// cf = coefficients field
	double Cu = 0, Cv = 0;
	double sumx, sumy, sumxy, cosu, cosv;
	double pi = Math.PI;
	double sqrt2 = Math.sqrt(2);
	int quantizeMode,tmpi;
	
		
	for (int v = 0; v < 8; v++)
	for (int u = 0; u < 8; u++)
	{
		if (v == 0) { Cv = 1/sqrt2; } else { Cv = 1; }
		if (u == 0) { Cu = 1/sqrt2; } else { Cu = 1; }
		
		sumxy = 0;
		for (int y = 0; y < 8; y++)
		{				
			for (int x = 0; x < 8; x++)
			{
				cosu = Math.cos( (2*x+1)*u*pi/16 );
				cosv = Math.cos( (2*y+1)*v*pi/16 );
				sumxy += (isf[y][x]-128)*cosu*cosv; // -128 -> zero shift
			} // end of for (int x ...
			
		} // end of for (int y ...
			
		cf[v][u] = (int)Math.round( sumxy*0.25*Cu*Cv ); // 0.25 = 0.5*0.5  ;-)
						
	} // end of outer double sum

		
	// quantize
	quantizeMode = gui.quantizeComboBox.getSelectedIndex();
		
	if (quantizeMode == 6) // JPEG Standard Table
	{ 			
		for (int u=0; u<=7; u++)
                for (int v=0; v<=7; v++)
		  	cf[u][v] = (int)( Math.round(cf[u][v] / m.StandardJPEGTable[u][v]) );
						
	} // end of if
	else 
	{	
		switch (quantizeMode)
		{	// quantize table with only one value in its cells
			case 0 : {tmpi = 1;break;}	// do not quantize
			case 1 : {tmpi = 2;break;}	// divide coeffs by tmpi
			case 2 : {tmpi = 5;break;}	// divide coeffs by tmpi
			case 3 : {tmpi = 10;break;}	// divide coeffs by tmpi
			case 4 : {tmpi = 25;break;}	// divide coeffs by tmpi
			case 5 : {tmpi = 50;break;}	// divide coeffs by tmpi
			default : {tmpi = 1;}
		} // end of switch
			
		for (int u=0; u<=7; u++)
		for (int v=0; v<=7; v++)
			cf[u][v] = (int)( Math.round(cf[u][v] / tmpi) ); 		
	} // end of else
		
	return cf;
		
} // end of public int[][] transformation( ... )
	
		
// ************************************** Backtransformation ****************************
public int[][] backtransformation(int[][] ucf) // ucf = user coefficients field
{			
	int[][] usf = new int[8][8]; // usf = user sample field
	double Cu = 0, Cv = 0;
	double sumu, sumv, cosu, cosv;
	double pi = Math.PI;
	double sqrt2 = Math.sqrt(2);
	int quantizeMode,tmpi;
				
		
	// dequantisation
		
	quantizeMode = gui.quantizeComboBox.getSelectedIndex();
		
	if (quantizeMode == 6) // JPEG Standard Table
	{ 			
		for (int u=0; u<=7; u++)
		for (int v=0; v<=7; v++)
		  	ucf[u][v] *= m.StandardJPEGTable[u][v];
			
	} // end of if
	else 
	{	
		switch (quantizeMode)
		{	// quantize table with only one value in its cells
			case 0 : {tmpi = 1;break;}	// do not quantize
			case 1 : {tmpi = 2;break;}	// divide coeffs by tmpi
			case 2 : {tmpi = 5;break;}	// divide coeffs by tmpi
			case 3 : {tmpi = 10;break;}	// divide coeffs by tmpi
			case 4 : {tmpi = 25;break;}	// divide coeffs by tmpi
			case 5 : {tmpi = 50;break;}	// divide coeffs by tmpi
			default : {tmpi = 1;}
		} // end of switch
			
		for (int u=0; u<=7; u++)
		for (int v=0; v<=7; v++)
		  	ucf[u][v] *= tmpi;
		
	} // end of else
		
		
	for (int y = 0; y < 8; y++)
	for (int x = 0; x < 8; x++)
	{
		sumv = 0;
		for (int v = 0; v < 8; v++)
		{				
			sumu = 0;				
			for (int u = 0; u < 8; u++)
			{
				if (u == 0) { Cu = 1/sqrt2;} else { Cu = 1;}
				cosu = Math.cos( (2*x+1)*u*pi/16 );
				cosv = Math.cos( (2*y+1)*v*pi/16 );
				sumu += 0.5*Cu*ucf[v][u]*cosu*cosv;
			} // end of for (u ...
			
			if (v == 0) { Cv = 1/sqrt2;} else { Cv = 1;}
			sumv += 0.5*Cv*sumu;				
		} // end of for (v ...
			
		usf[y][x] = (int)Math.round(sumv)+128; // +128 = zero shift
			
		//if (usf[y][x] < 0)   usf[y][x] = 0;
		//if (usf[y][x] > 255) usf[y][x] = 255;			
			
	} // end of outer double sum
				
	return usf;
		
} // end of public int[][] backtransformation( ... )
	
				
// ************************************** Calculate *************************************				
public void calculate(String calculationmode)
{			
	if (calculationmode.equals("signals") || calculationmode.equals("all"))
	{
		inputSampleToTransform = gui.getInputSampleToUse();
		coefField = transformation(inputSampleToTransform);			
	}
		
	if (calculationmode.equals("coefficients") || calculationmode.equals("all"))
	{							   
		userCoefSampleToTransformBack = gui.getCoefValuesFromTextfields();		
		userInputSample = backtransformation(userCoefSampleToTransformBack); 					
	}	
				
	gui.updateGrayscalePanels(inputSampleToTransform,userInputSample);
					
} // end of public void calculate()

} // end of class