// TheEventHandler

import java.awt.*;
import java.awt.event.*;
import java.awt.Color.*;
import javax.swing.*;

class TheEventHandler implements ActionListener, AdjustmentListener, FocusListener
{
	
TheFrame frame;
TheGUI gui;
TheCalculator calculator;

int oldx = 0, oldy = 0, actx = 0, acty = 0;
	
TheEventHandler(TheFrame frame,
		TheGUI gui, 
		TheCalculator calculator)
{
	this.frame = frame;
	this.gui = gui;	
	this.calculator = calculator;
	
} // end of constructor


// ************************************** Action Performed *********************************************	
	
public void actionPerformed(ActionEvent evt)
{	
	
	//******************************** COMBOBOXES ****************************
	if (evt.getSource() instanceof JComboBox)
	{		
		// the reaction to the quantizeComboBox
		if (evt.getSource() == gui.quantizeComboBox)
		{	// Hier ist eigentlich nichts zu tuen, 
			//   da in den Transformationsmethoden der Index abgefragt wird
			JComboBox source = (JComboBox)evt.getSource();
			int selectedQuantizeMode = (int)source.getSelectedIndex();				
		} else		
		// the reaction to the inputsampleComboBox
		if (evt.getSource() == gui.inputsampleComboBox) 
		{
			gui.setInputSampleToUse();
			calculator.inputSampleToTransform = gui.getInputSampleToUse();	
			calculator.calculate("all");		
		}				
	} // end ComboBox Handler
	
	
	//******************************** TEXTFIELDS ****************************
	if (evt.getSource() instanceof JTextField2)
	{
		JTextField2 source = (JTextField2)evt.getSource();		
		
		for (int y = 0; y < 8; y++)
		for (int x = 0; x < 8; x++)
			if (!(source == gui.coefTextFieldMatrix[y][x]))
				continue;
			else  
			{ 
				if (!m.isValid(source.getText().trim(),m.minAmplDCT,m.maxAmplDCT))
					source.setText("0");	
				gui.coefScroller.setValue(Integer.parseInt(source.getText().trim())*-1);
				oldy = acty;
				oldx = actx;
				acty = y;
				actx = x;
				break;
			} // end of else
		
		calculator.calculate("all");
	} // end of jtextfield handler
		
		
	//******************************** BUTTONS ****************************		
	if (evt.getSource() instanceof JButton) 
	{	
		// set all coefficients to zero
		if ((JButton)evt.getSource() == gui.resetCoefficientsButton)
		{
			for (int y = 0; y < 8; y++)
			for (int x = 0; x < 8; x++)
				  gui.coefTextFieldMatrix[y][x].setText("0");			  
			gui.coefScroller.setValue(0);
			calculator.calculate("all");
		} else
		// show the quantize table
		if ((JButton)evt.getSource() == gui.showQuantizeTableButton)
		{
			JDialog d = new QuantizeDialog(frame, gui.quantizeComboBox.getSelectedIndex());
			d.show();
		} else
		// show an dialog window where the user can customize an input pattern
		if ((JButton)evt.getSource() == gui.inputCustomizerButton)
		{
			int[][] dummy = gui.getCustomizedInput();
			int l = gui.inputSamplesField.length-1; // get the amount of avail. input patterns
			System.out.println(l);
			
			// dummy equals 'null' when user pressed the 'cancel' button in the inputchoser dialog
			// when this happens -> nothing else should happen
			if (dummy != null)
			{
				for (int y = 0; y < 8; y++)
				for (int x = 0; x < 8; x++)
					gui.inputSamplesField[l][y][x]=dummy[y][x];
				gui.inputsampleComboBox.setSelectedIndex(l);
				gui.setInputSampleToUse();
				calculator.inputSampleToTransform = gui.getInputSampleToUse();
								
				calculator.calculate("all");
			}
		}			
	}

		
	//******************************** MENU OBJECTS ****************************		
	if (evt.getSource() instanceof JMenuItem)
	{	
		String arg = evt.getActionCommand();
		
		// the reaction to the 'magic' option
		if (arg.equals("** MAGIC **")) 
		{		
			gui.coefTextFieldMatrix[acty][actx].setBackground(Color.white);
	  		calculator.calculate("signals");
	  				  		
			for (int y = 0; y < 8; y++)
			for (int x = 0; x < 8; x++)
				gui.coefTextFieldMatrix[y][x].setText(String.valueOf(calculator.coefField[y][x]));
											
			calculator.calculate("coefficients");
			
			gui.coefTextFieldMatrix[0][0].requestFocus();
			gui.coefScroller.setValue(Integer.parseInt(gui.coefTextFieldMatrix[0][0].getText().trim())*-1);
			oldy = acty = 0;
			oldx = actx = 0;
			gui.coefTextFieldMatrix[acty][actx].setBackground(Color.yellow);
			
		}
		else		
		if (arg.equals("Help")) 
		{	String message;
		
			message = 
			"This program visualizes the effect of the\n"+
			"2-Dimensional Discrete Cosine Transform (DCT).\n"+
			"One the left hand you see the image space and on the\n"+
			"right hand the  frequency space. The examples represent\n"+
			"target grayscales that you should try to approximate\n"+
			"by putting correct coefficients on the frequencies.\n"+
			"The Inverse Discrete Cosine Transform (IDCT) takes\n"+
			"the coefficient values of the right hand side and visualizes\n"+
			"the image at \"Your approximation\". You should try to\n"+
			"solve the examples in the proposed order - it gets more\n"+
			"and more complex with the time. You can also customize\n"+
			"the target image by clicking on \"Customize input\" and\n"+
			"selecting the grayscale values on your own. As the DCT is\n"+
			"used in many image processing algorithms, such as JPEG, we\n"+
			"have included the possibility to quantize the coefficients -\n"+
			"and even to chose a quantization table. A solution can\n"+
			"be found at ** Magic **.\n\n"+
			"Please mail comments to\n"+
			"schremmer@informatik.uni-mannheim.de";				  
			
			JOptionPane.showMessageDialog(frame,message,"About",JOptionPane.INFORMATION_MESSAGE);
		}
	} // end of jmenuitem handler
	
} // end of public void ActionPerformed(Action evt)


// ************************************** Adjustment Value Changed *****************************************

public void adjustmentValueChanged(AdjustmentEvent evt)
{	
	String tmps = "";
	JScrollBar2 source = (JScrollBar2)evt.getSource();
		
	// the reaction to the adjustment of the coefScroller
	if (source == gui.coefScroller)
	{	
		tmps = String.valueOf(source.getValue()*-1); // value of the coefScroller		
		gui.coefTextFieldMatrix[acty][actx].setText(tmps);			
	}
		
	calculator.calculate("all");
		
			
} // end of public void adjustment value changed(...
	
// ************************************** FocusEvent occured *****************************************	
public void focusLost(FocusEvent evt)
{	
	if ((evt.getSource() instanceof JTextField2) && (!evt.isTemporary()))	
	{
		JTextField2 source = (JTextField2)evt.getSource();			
		// the reaction when leaving a textfield
		
		for (int y = 0; y < 8; y++)
		for (int x = 0; x < 8; x++)
			if (!(source == gui.coefTextFieldMatrix[y][x]))
				continue;
			else				
			{ 
				if (!m.isValid(source.getText().trim(),m.minAmplDCT,m.maxAmplDCT))
				{
					source.setText("0");	
					source.requestFocus();
					gui.coefTextFieldMatrix[y][x].setBackground(Color.white);
				}				
				calculator.calculate("all");
				break;				
			} // end of else
	} // end of outer if			
	
} // end of public void focusLost(FocusEvent evt)
	
	
public void focusGained(FocusEvent evt)
{ 	
	JTextField2 source = (JTextField2)evt.getComponent();	
	source.selectAll();
	String tmps;
	int tmpi;
	
	
		
	for (int y = 0; y < 8; y++)
	for (int x = 0; x < 8; x++)
	  	if (!(source == gui.coefTextFieldMatrix[y][x]))
	  		gui.coefTextFieldMatrix[y][x].setBackground(Color.white);
	  	else 
	  	{ 
	  		oldy = acty;
			oldx = actx;
	  		acty = y; 
	  		actx = x; 
	  		gui.coefScroller.setValue(Integer.parseInt(source.getText().trim())*-1);
	  		
	  		gui.coefTextFieldMatrix[oldy][oldx].setBackground(Color.white);	
			gui.coefTextFieldMatrix[acty][actx].setBackground(Color.yellow);
			gui.coefScroller.setToolTipText("Connected to coefficient U"+acty+actx); 
	  	} // end of else	
		
}// end of public void focusGained(FocusEvent evt)

} // end of class